;;; -*- Mode: Emacs-Lisp -*-
;;;
;;; (C) 1997 mrz@informatik.uni-jena.de | www2.informatik.uni-jena.de/~mrz/
;;; Colorizes the lines in the Dialogue buffer

(eval-when-compile (require 'irchat-inlines))
(eval-when-compile (require 'font-lock))

;; regular expressions for many things:

(defvar irchat-nick-expr          "^\\( \\|<\\)*[\]a-z0-9_\\^`'[{}|\\-]+:"
  "Regular Expression to recognize nicknames.
Needs the Value \"%9s:\" for irchat-format-string2!")

(defvar irchat-email-expr         "[-.a-z0-9~}{_]+@[-a-z0-9.}{_][-a-z0-9.}{_]+"
  "Regular Expression to recognize email addresses")

(defvar irchat-string-expr        "\"[^\"]+\""
  "Regular Expression to recognize strings")

(defvar irchat-channelname-expr   "#[.a-z0-9/_%-]+\\( \\|$\\|>\\)"
  "Regular Expression to recognize channel names")

(defvar irchat-irc-cmds-expr      " /[a-z]+\\( \\|$\\)"
  "Regular Expression to recognize irc commands")

(defvar irchat-URL-expr           "http://[a-z0-9~./?_-]+"
  "Regular Expression to recognize URL's")

(defvar irchat-FTP-URL-expr       "ftp://[a-z0-9~./_-]+"
  "Regular Expression to recognize ftp-URL's")

(defvar irchat-fun-expr           " \\*[^\\*]+\\*\\( \\|$\\)"
  "Regular Expression to recognize *fun*")

(defvar irchat-privmsg-expr      "^*[\]a-z0-9_\\^`'[{}|\\-]+\\* "
  "Regular Expression to recognize private messages")

(defvar irchat-bold1-expr         "\\[^\\]+\\"
  "Regular Expression to recognize (real) ^B...^B")

(defvar irchat-bold2-expr         "\\^B[^\\^B]+\\^B"
  "Regular Expression to recognize (2 chars) ^B...^B")

(defvar irchat-underline1-expr    "\\[^\\]+\\"
  "Regular Expression to recognize (real) ^_...^_")

(defvar irchat-underline2-expr    "\\^_[^\\^_]+\\^_"
  "Regular Expression to recognize (2 chars) ^_...^_")


(defconst irchat-color-expface-list
  (list
   (list 'irchat-nick-expr        'font-lock-type-face)
   (list 'irchat-string-expr      'font-lock-string-face)
   (list 'irchat-email-expr       'font-lock-preprocessor-face)
   (list 'irchat-channelname-expr 'font-lock-keyword-face)
   (list 'irchat-irc-cmds-expr    'font-lock-keyword-face)
   (list 'irchat-URL-expr         'font-lock-preprocessor-face)
   (list 'irchat-FTP-URL-expr     'font-lock-preprocessor-face)
   (list 'irchat-fun-expr         'font-lock-variable-name-face)
   (list 'irchat-privmsg-expr     'font-lock-preprocessor-face)
;;   (list  irchat-real-nickname    'warning)
;;  (list irchat-real-nickname     'font-lock-preprocessor-face)
   )
  "List of regular expressions with corresponding faces.
The coloring will be performed in the above order
(eg: strings will be colored before emails and so on)")


(defconst irchat-color-expfacedel-list
  (list
   (list 'irchat-bold1-expr       'bold      1)
   (list 'irchat-bold2-expr       'bold      2)
   (list 'irchat-underline1-expr  'underline 1)
   (list 'irchat-underline2-expr  'underline 2)
   )
  "List of regular expressions with corresponding faces.
This expressions are bounded be chars like ^B, which will be deleted.
The third parameter is the count of chars of this border.")




(defun irchat-color-whole-line-with-reg-exp ( expr face )
  "If line matches expr, colorise whole line with face."
  (interactive)
  (beginning-of-line)
  (if(search-forward-regexp expr nil t)
      (set-extent-face (make-extent (match-beginning 0) (match-end 0)) face)
    nil)
)

(defun irchat-color-reg-exp ( expr face )
  "If a part of the line matches expr, colorise this part with face."
  (interactive)
  (beginning-of-line)
  (while(search-forward-regexp expr nil t)
    (set-extent-face (make-extent (match-beginning 0) (match-end 0)) face)
    nil)
)


(defun irchat-color-reg-exp+del ( expr face len)
  "If a part of the line matches expr, colorise this part with face.
This expressions are bounded be chars like ^B, which will be deleted.
The parameter lenght is the count of chars of this border."
  (interactive)
  (beginning-of-line)
  (while(search-forward-regexp expr nil t)
    (delete-char (- len))
    (goto-char (match-beginning 0))
    (delete-char len)
    (set-extent-face
     (make-extent (match-beginning 0) (- (match-end 0) (+ len len)))
     face)
    (goto-char (- (match-end 0) (+ len len)))
    nil)
)


(defun irchat-colors-by-mrz-hook ()
  "Hook used to colorize the lines in the Dialogue buffer."
  (interactive)
  
  (previous-line 1) (beginning-of-line)
  
  ;; whole lines:
  (irchat-color-whole-line-with-reg-exp "^\*[*][*].*$"  'font-lock-comment-face)
  (irchat-color-whole-line-with-reg-exp "^> .*$"        'red)
  (irchat-color-whole-line-with-reg-exp "^-> .*$"       'red)
  ;;(irchat-color-whole-line-with-reg-exp "^\\[.*\\]$"    'font-lock-comment-face)

  ;; now expressions:
  ;;  (let ( (l irchat-color-expface-list) (v nil))
  (let ( (l irchat-color-expface-list) )
    (while (not (null l))
      ;;(setq v (car l))
      (irchat-color-reg-exp (eval(caar l)) (eval(cadar l)) )
      (setq l (cdr l))
      )
    )
  
  (irchat-color-reg-exp+del irchat-bold1-expr       'bold 1)
  (irchat-color-reg-exp+del irchat-bold2-expr       'bold 2)
  (irchat-color-reg-exp+del irchat-underline1-expr  'underline 1)
  (irchat-color-reg-exp+del irchat-underline2-expr  'underline 2)

  (irchat-color-reg-exp irchat-real-nickname 'red)

  ;; go back to the beginning, so irc-insert will not get confused:
  (goto-char (point-max))
  )

(provide 'irchat-colors-by-mrz)
